# ------------------------------------------------------------------------------#
#  CCS: Curvature Constrained Splines                                          #
#  Copyright (C) 2019 - 2023  CCS developers group                             #
#                                                                              #
#  See the LICENSE file for terms of usage and distribution.                   #
# ------------------------------------------------------------------------------#

"""
Functionality to fit curvature constraint splines.
"""


import logging
import argparse
import os

from ccs_fit.fitting.main import twp_fit

FILENAME = "CCS_input.json"


USAGE = """
A tool to fit two body potentials using constrained cubic splines.
"""


def main(cmdlineargs=None):
    """Main driver routine for ccs_fit.

    Args:

        cmdlineargs: list of command line arguments
            When None, arguments in sys.argv are parsed (default: None)

    """

    size = os.get_terminal_size()
    c = size.columns
    txt = "-"*c
    print("")
    print(txt)

    try:
        import art
        txt = art.text2art('CCS:Fit')
        print(txt)
    except:
        pass

    args = parse_cmdline_args(cmdlineargs)
    ccs_fit(args)

    size = os.get_terminal_size()
    c = size.columns
    txt = "-"*c
    print(txt)
    print("")


def parse_cmdline_args(cmdlineargs=None):
    """Parses command line arguments.

    Args:

        cmdlineargs: list of command line arguments
            When None, arguments in sys.argv are parsed (default: None)

    """

    parser = argparse.ArgumentParser(description=USAGE)

    msg = "Json file containing pairwise distances and energies"
    parser.add_argument("input", nargs="?", default=FILENAME, help=msg)

    msg = "Log level for debugging"
    parser.add_argument(
        "-d",
        "--debug",
        dest="loglvl",
        default=logging.INFO,
        const=logging.DEBUG,
        action="store_const",
        help=msg,
    )

    args = parser.parse_args(cmdlineargs)

    fmt = "%(asctime)s - %(name)s - %(levelname)s -       %(message)s"
    logging.basicConfig(filename="CCS.log", format=fmt, level=args.loglvl)

    return args


def ccs_fit(args):
    """Reads desired output file and generates a structures.json file.

    Args:

        args: namespace of command line arguments

    """

    logging.info("Started")
    twp_fit(args.input)
    logging.info("Ended")



import sys
import os
import numpy as np
import ase.db as db
import json
import random
from ase.calculators.singlepoint import SinglePointCalculator
from tqdm import tqdm

from ccs_fit.ase_calculator.ccs_ase_calculator import CCS


def ccs_validate(
    mode=None,
    CCS_params=None,
    Ns="all",
    DFT_DB=None,
    CCS_DB="CCS_validate.db",
    DFTB_DB=None,
    charge=False,
    q=None,
    charge_scaling=False,
):
    """
    Function to verify database generation.
    ---------------------------------------

    Input
    -----
        mode : str
            String describing which mode is used, supported modes are:
                CCS:   CCS_params_file(string) NumberOfSamples(int) DFT.db(string)")
                CCS+Q: CCS_params_file(string) NumberOfSamples(int) DFT.db(string) charge_dict(string) charge_scaling(bool)")
                DFTB:  CCS_params_file(string) NumberOfSamples(int) DFT.db(string) DFTB.db(string)")

    Returns
    -------
        What does it return Jolla?

    Example usage
    -------------
        ccs_validate MODE [...]
    """
    if os.path.isfile(CCS_DB):
        os.remove(CCS_DB)

    DFT_DB = db.connect(DFT_DB)
    CCS_DB = db.connect(CCS_DB)
    if mode == "DFTB":
        DFTB_DB = db.connect(DFTB_DB)

    f = open("CCS_validate.dat", "w")
    print("#Reference      Predicted      Error      No_of_atoms structure_no", file=f)

    calc = CCS(CCS_params=CCS_params, charge=charge,
               q=q, charge_scaling=charge_scaling)

    if Ns != "all":
        Ns = int(Ns)
        mask = [a <= Ns for a in range(len(DFT_DB))]
        random.shuffle(mask)
    else:
        mask = len(DFT_DB) * [True]

    counter = -1
    for row in tqdm(DFT_DB.select(), total=len(DFT_DB), colour="#800000"):
        counter += 1
        if mask[counter]:
            structure = row.toatoms()
            EDFT = structure.get_total_energy()
            EREF = EDFT
            structure.calc = calc
            ECCS = structure.get_potential_energy()
            if mode == "DFTB":
                key = row.key
                EDFTB = DFTB_DB.get("key=" + str(key)).energy
                EREF = EDFT - EDFTB
                sp_calculator = SinglePointCalculator(
                    structure, energy=EDFTB + ECCS)
                structure.calc = sp_calculator
                structure.get_potential_energy()

            print(EREF, ECCS, np.abs(EREF - ECCS),
                  len(structure), counter, file=f)
            try:
                CCS_DB.write(structure, key=key)
            except:
                CCS_DB.write(structure)


def main():
    size = os.get_terminal_size()
    c = size.columns
    txt = "-"*c
    print("")
    print(txt)

    try:
        import art
        txt = art.text2art('CCS:Validate')
        print(txt)
    except:
        pass

    print("    USAGE:  ccs_validate MODE [...] ")
    print(" ")
    print("    The following modes and inputs are supported:")
    print("")
    print("        CCS:   CCS_params_file(string) NumberOfSamples(int) DFT.db(string)")
    print(
        "       CCS+Q: CCS_params_file(string) NumberOfSamples(int) DFT.db(string) charge_dict(string) charge_scaling(bool)"
    )
    print(
        "        DFTB:  CCS_params_file(string) NumberOfSamples(int) DFT.db(string) DFTB.db(string)"
    )
    print("")

    try:
        assert sys.argv[1] in ["CCS", "CCS+Q", "DFTB"], "Mode not supproted."
    except:
        exit()

    mode = sys.argv[1]
    CCS_params_file = sys.argv[2]
    Ns = sys.argv[3]
    DFT_data = sys.argv[4]
    with open(CCS_params_file, "r") as f:
        CCS_params = json.load(f)

    print("    Mode: ", mode)
    if mode == "CCS":
        print("    Number of samples: ", Ns)
        print("    DFT reference data base: ", DFT_data)
        print("")

        ccs_validate(mode=mode, CCS_params=CCS_params, Ns=Ns, DFT_DB=DFT_data)
    if mode == "DFTB":
        DFTB_data = sys.argv[5]
        print("    Number of samples: ", Ns)
        print("    DFT reference data base: ", DFT_data)
        print("    DFTB reference data base: ", DFTB_data)
        print("")

        ccs_validate(
            mode=mode, CCS_params=CCS_params, Ns=Ns, DFT_DB=DFT_data, DFTB_DB=DFTB_data
        )
    if mode == "CCS+Q":
        print(
            "        NOTE: charge_dict should use double quotes to enclose property nanes. Example:"
        )
        print('        \'{"Zn":2.0,"O" : -2.0 } \'')
        charge_dict = sys.argv[5]
        charge_scaling = sys.argv[6]
        if charge_scaling == "True":
            charge_scaling = True
        if charge_scaling == "False":
            charge_scaling = False
        print("    Number of samples: ", Ns)
        print("    DFT reference data base: ", DFT_data)
        print("    Charge dictionary: ", charge_dict)
        print("    Charge scaling: ", charge_scaling)
        print("")

        charge_dict = json.loads(charge_dict)
        ccs_validate(
            mode=mode,
            CCS_params=CCS_params,
            Ns=Ns,
            DFT_DB=DFT_data,
            charge=True,
            q=charge_dict,
            charge_scaling=charge_scaling,
        )


if __name__ == "__main__":
    main()



# ------------------------------------------------------------------------------#
#  CCS: Curvature Constrained Splines                                          #
#  Copyright (C) 2019 - 2023  CCS developers group                             #
#                                                                              #
#  See the LICENSE file for terms of usage and distribution.                   #
# ------------------------------------------------------------------------------#

import logging
import numpy as np
import itertools as it
from collections import OrderedDict, defaultdict
from numpy import linalg as LA
from ase.calculators.calculator import Calculator, all_changes
from ase.constraints import full_3x3_to_voigt_6_stress

try:
    from pymatgen.core import Lattice, Structure
    from pymatgen.analysis import ewald
except:
    pass

logging.basicConfig(filename="ccs.spl", level=logging.DEBUG)
logg = logging.getLogger(__name__)


class spline_table:
    def __init__(self, elem1, elem2, CCS_params, exp=True):
        self.elem1 = elem1
        self.elem2 = elem2
        self.no_pair = False
        try:
            pair = elem1 + "-" + elem2
            self.rcut = CCS_params["Two_body"][pair]["r_cut"]
        except:
            try:
                pair = elem2 + "-" + elem1
                self.rcut = CCS_params["Two_body"][pair]["r_cut"]
            except:
                self.rcut = 0.0
                self.no_pair = True
        if self.no_pair:
            self.a = [0.0]
            self.b = [0.0]
            self.c = [0.0]
            self.d = [0.0]
            self.aa = 0.0
            self.bb = 0.0
            self.cc = 0.0
            self.Rmin = 0.0
            self.Rcut = 0.0
            self.dx = 1.0
            self.x = [0.0]
            self.exp = False
        else:
            self.Rmin = CCS_params["Two_body"][pair]["r_min"]
            self.Rcut = CCS_params["Two_body"][pair]["r_cut"]
            self.a = CCS_params["Two_body"][pair]["spl_a"]
            self.b = CCS_params["Two_body"][pair]["spl_b"]
            self.c = CCS_params["Two_body"][pair]["spl_c"]
            self.d = CCS_params["Two_body"][pair]["spl_d"]
            self.aa = CCS_params["Two_body"][pair]["exp_a"]
            self.bb = CCS_params["Two_body"][pair]["exp_b"]
            self.cc = CCS_params["Two_body"][pair]["exp_c"]
            self.exp = False
            self.x = CCS_params["Two_body"][pair]["r"]
            self.dx = CCS_params["Two_body"][pair]["dr"]

    def eval_energy(self, r):
        index = int(np.floor((r - self.Rmin) / self.dx))

        if r >= self.Rmin and r <= self.rcut:
            dr = r - self.x[index]
            f0 = self.a[index] + dr * (
                self.b[index] + dr * (self.c[index] + (self.d[index] * dr))
            )
            return float(f0)
        elif r < self.Rmin:
            val = np.exp(-self.aa * r + self.bb) + self.cc
            return val
        else:
            val = 0.0
            return val

    def eval_force(self, r):

        index = int(np.floor((r - self.Rmin) / self.dx))

        if r >= self.Rmin and r <= self.rcut:
            dr = r - self.x[index]
            f1 = self.b[index] + dr * (2 * self.c[index] + (3 * self.d[index] * dr))
            return f1
        elif r < self.Rmin:
            val = -self.aa * np.exp(-self.aa * r + self.bb)
            return val
        else:
            val = 0.0
            return val


def ew(atoms, q):

    #   structure = AseAtomsAdaptor.get_structure(atoms)
    atoms.charges = []
    for a in atoms.get_chemical_symbols():
        atoms.charges.append(q[a])
    lattice = Lattice(atoms.get_cell())
    coords = atoms.get_scaled_positions()
    struct = Structure(
        lattice,
        atoms.get_chemical_symbols(),
        coords,
        site_properties={"charge": atoms.charges},
    )
    Ew = ewald.EwaldSummation(struct, compute_forces=True)
    return Ew


class CCS(Calculator):
    """
    CCS calculator

    Curvature constrained splines calculator compatible with the ASE
    format.

    Parameters
    ----------
    input_file : XXX
        To be added, Jolla.

    Returns
    -------
    What does it return actually, Jolla?


    Examples
    --------
    >>> To be added, Jolla.
    """

    implemented_properties = {"stress", "energy", "forces"}

    def __init__(
        self, CCS_params=None, charge=None, q=None, charge_scaling=False, **kwargs
    ):
        self.rc = 7.0  # SET THIS MAX OF ANY PAIR
        self.exp = None
        self.charge = charge
        self.species = None
        self.pair = None
        self.q = q
        self.CCS_params = CCS_params
        self.eps = CCS_params["One_body"]
        if charge_scaling:
            for key in self.q:
                self.q[key] *= self.CCS_params["Charge scaling factor"]

        Calculator.__init__(self, **kwargs)

    def calculate(self, atoms=None, properties=["energy"], system_changes=all_changes):
        Calculator.calculate(self, atoms, properties, system_changes)
        self.species = list(set(self.atoms.get_chemical_symbols()))
        self.pair = dict()
        for a, b in it.product(self.species, self.species):
            self.pair[a + b] = spline_table(a, b, self.CCS_params)

        if self.atoms.number_of_lattice_vectors == 3:
            cell = atoms.get_cell()
            n_repeat = self.rc * np.linalg.norm(np.linalg.inv(cell), axis=0)
            n_repeat = np.ceil(n_repeat).astype(int)
            offsets = [*it.product(*[np.arange(-n, n + 1) for n in n_repeat])]

        natoms = len(self.atoms)
        dict_species = defaultdict(int)
        for elem in self.atoms.get_chemical_symbols():
            dict_species[elem] += 1

        energy = 0.0
        forces = np.zeros((natoms, 3))
        stresses = np.zeros((natoms, 3, 3))

        # ONE-BODY ENERGY
        elems = it.combinations_with_replacement(dict_species.keys(), 1)
        for elem in elems:
            try:
                energy += self.eps[elem[0]] * dict_species[elem[0]]
            except:
                pass

        # PAIR-WISE ENERGY AND FORCE
        for x, y in it.product(self.species, self.species):
            xy_distances = []
            mask1 = [atom == x for atom in self.atoms.get_chemical_symbols()]
            mask2 = [atom == y for atom in self.atoms.get_chemical_symbols()]
            pos1 = self.atoms[mask1].positions
            index1 = np.arange(0, len(self.atoms))[mask1]
            atoms_2 = self.atoms[mask2]
            if self.atoms.number_of_lattice_vectors == 3:
                pos2 = []
                for offset in offsets:
                    pos2.append((atoms_2.positions + offset @ cell))
            else:
                pos2 = list(atoms_2.positions)
            pos2 = np.array(pos2)
            pos2 = np.reshape(pos2, (-1, 3))
            for p1, id in zip(pos1, index1):
                dist = pos2 - p1
                norm_dist = np.linalg.norm(dist, axis=1)
                dist_mask = (norm_dist < self.rc) & (norm_dist > 0)
                xy_distances.extend(norm_dist[dist_mask].tolist())
                # Sometimes there are no distances to append

                try:
                    forces[id, :] += np.sum(
                        (
                            dist[dist_mask].T
                            * list(
                                map(self.pair[x + y].eval_force, norm_dist[dist_mask])
                            )
                            / norm_dist[dist_mask]
                        ).T,
                        axis=0,
                    )
                except:
                    pass

            energy += 0.5 * sum(map(self.pair[x + y].eval_energy, xy_distances))

        if self.charge:
            ewa = ew(self.atoms, self.q)
            energy = energy + ewa.total_energy
            forces = forces + ewa.forces

        self.results["energy"] = energy
        self.results["free_energy"] = energy
        self.results["forces"] = forces

        if self.atoms.number_of_lattice_vectors == 3:
            stresses = full_3x3_to_voigt_6_stress(stresses)
            self.results["stress"] = stresses.sum(axis=0) / self.atoms.get_volume()
            self.results["stresses"] = stresses / self.atoms.get_volume()

        # natoms = len(self.atoms)
        # if 'numbers' in system_changes:
        #     self.nl = NeighborList(
        #         [self.rc / 2] * natoms, bothways=True, self_interaction=False)

        # self.nl.update(self.atoms)

        # positions = self.atoms.positions
        # cell = self.atoms.cell

        # for at in range(natoms):
        #     elem1 = self.atoms.get_chemical_symbols()[at]
        #     if self.eps is not None:
        #         try:
        #             energy_eps = energy_eps + self.eps[elem1]
        #         except:
        #             pass
        #     indices, offsets = self.nl.get_neighbors(at)
        #     f_tot = np.zeros((1, 3))
        #     s_tot = np.zeros((3, 3))

        #     for i, offset in zip(indices, offsets):
        #         elem2 = self.atoms.get_chemical_symbols()[i]
        #         d_vector = positions[i] + np.dot(offset, cell) - positions[at]
        #         d = np.linalg.norm(d_vector)
        #         energy += self.pair[elem1+elem2].eval_energy(d)
        #         # check this
        #         f = self.pair[elem1+elem2].eval_force(d)*(d_vector/d)
        #         s_tot += 0.5 * np.outer(f, d_vector)
        #         f_tot += f
        #     forces[at] = f_tot
        #     stresses[at] = s_tot
        # energy = 0.5*energy  # Only bothways true

        # # IF WE HAVE A LATTICE DEFINE STRESS
        # if self.atoms.number_of_lattice_vectors == 3:
        #     stresses = full_3x3_to_voigt_6_stress(stresses)
        #     self.results['stress'] = (
        #         stresses.sum(axis=0) / self.atoms.get_volume()
        #     )
        #     self.results['stresses'] = stresses / self.atoms.get_volume()

        # energy = energy + energy_eps
        # if self.charge:
        #     ewa = ew(self.atoms, self.q)
        #     energy_ccs = energy
        #     energy = energy + ewa.total_energy
        #     forces_ccs = forces
        #     forces = forces + ewa.forces



# ------------------------------------------------------------------------------#
#  CCS: Curvature Constrained Splines                                          #
#  Copyright (C) 2019 - 2023  CCS developers group                             #
#                                                                              #
#  See the LICENSE file for terms of usage and distribution.                   #
# ------------------------------------------------------------------------------#

"""
Functionality to derive spline table for LAMMPS
"""

import json
import sys
import itertools
import numpy as np
from collections import OrderedDict
from ccs_fit.ase_calculator.ccs_ase_calculator import spline_table


def asecalcTotable(jsonfile, scale=10, table="CCS.table"):
    json_file = open(jsonfile)
    CCS_params = json.load(json_file)
    energy = []
    force = []
    tags = {}
    with open(table, "w") as f:
        for pair in CCS_params["Two_body"].keys():
            elem1, elem2 = pair.split("-")
            tb = spline_table(elem1, elem2, CCS_params)
            rmin=CCS_params["Two_body"][pair]["r_min"]
            dr = CCS_params["Two_body"][pair]["dr"] / scale
            r = np.arange(rmin, tb.Rcut + dr, dr)
            tags[pair]=dict({'Rmin':rmin,'Rcut':tb.Rcut,'dr':dr,'N':len(r)})
            f.write("\n {}".format(pair))
            f.write("\n N {} R {} {} \n".format(len(r), rmin, tb.Rcut))
            [
                f.write(
                    "\n {} {} {} {}".format(
                        index + 1, elem, tb.eval_energy(elem), -1 * tb.eval_force(elem)
                    )
                )
                for index, elem in enumerate(r)
            ]

    return tags



if __name__ == "__main__":
    f = sys.argv[1]
    asecalcTotable(f)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ccs_fit',
 'ccs_fit.ase_calculator',
 'ccs_fit.common',
 'ccs_fit.common.math',
 'ccs_fit.data',
 'ccs_fit.debugging_tools',
 'ccs_fit.fitting',
 'ccs_fit.ppmd_interface',
 'ccs_fit.regression_tool',
 'ccs_fit.scripts']

package_data = \
{'': ['*']}

install_requires = \
['ase>=3.22.1',
 'cvxopt>=1.3.0',
 'numpy>=1.22.4,<1.23.0',
 'pandas>=1.5.0',
 'scipy>=1.9.2',
 'sympy>=1.11.1',
 'tqdm>=4.64.1']

setup_kwargs = {
    'name': 'ccs-fit',
    'version': '0.11.17',
    'description': 'Fitting tools for repulsive two body interactions using curvature constrained splines.',
    'long_description': '# CCS - Curvature Constrained Splines  \n\n[![Latest Release](https://img.shields.io/github/v/release/teoroo-cmc/ccs?display_name=tag&color=brightgreen&sort=semver)](https://github.com/Teoroo-CMC/CCS/releases/latest)\n[![License](https://img.shields.io/github/license/teoroo-cmc/ccs)](https://opensource.org/licenses/LGPL-3.0)\n[![DOI](https://img.shields.io/badge/DOI-10.1016%2Fj.cpc.2020.107602-blue)](https://doi.org/10.1016/j.cpc.2020.107602)\n[![Build](https://img.shields.io/github/actions/workflow/status/teoroo-cmc/CCS/ci-cd.yml)](https://github.com/Teoroo-CMC/CCS/actions)\n\n<!--- [![Build Status](https://github.com/tblite/tblite/workflows/CI/badge.svg)](https://github.com/tblite/tblite/actions)\n[![Documentation](https://img.shields.io/badge/Github%20Pages-Pages-blue)](https://teoroo-cmc.github.io/CCS/)\n[![codecov](https://codecov.io/gh/tblite/tblite/branch/main/graph/badge.svg?token=JXIE6myqNH)](https://codecov.io/gh/tblite/tblite) \n[![Coverage](codecov.io/gh/:vcsName/:user/:repo?flag=flag_name&token=a1b2c3d4e5)(https://github.com/Teoroo-CMC/CCS/actions)\n--->\n\nThe `CCS` package is a tool to construct two-body potentials using the idea of curvature constrained splines.\n## Getting Started\n### Package Layout\n\n```\nccs-x.y.z\n├── CHANGELOG.md\n├── LICENSE\n├── MANIFEST.in\n├── README.md\n├── bin\n│\xa0\xa0 ├── ccs_build_db\n│\xa0\xa0 ├── ccs_export_sktable\n│\xa0\xa0 ├── ccs_fetch\n│\xa0\xa0 ├── ccs_fit\n│\xa0\xa0 └── ccs_validate\n├── docs\n├── examples\n│\xa0\xa0 ├── CCS\n│\xa0\xa0 ├── CCS_with_LAMMPS\n│\xa0\xa0 ├── DFTB_repulsive_fitting\n│\xa0\xa0 ├── Preparing_ASE_db_trainingsets\n│\xa0\xa0 ├── Simple_regressor\n│\xa0\xa0 ├── Twobody_fit_for_an_O2_molecule\n│\xa0\xa0 ├── Twobody_fit_for_solid_Ne\n│\xa0\xa0 └── ppmd_interfacing\n├── logo.png\n├── poetry.lock\n├── pyproject.toml\n├── src\n│\xa0\xa0 └── ccs\n│\xa0\xa0     ├── ase_calculator\n│\xa0\xa0     ├── common\n│\xa0\xa0     ├── data\n│\xa0\xa0     ├── debugging_tools\n│\xa0\xa0     ├── fitting\n│\xa0\xa0     ├── ppmd_interface\n│\xa0\xa0     ├── regression_tool\n│\xa0\xa0     └── scripts\n│\xa0\xa0         ├── ccs_build_db.py\n│\xa0\xa0         ├── ccs_export_sktable.py\n│\xa0\xa0         ├── ccs_fetch.py\n│\xa0\xa0         ├── ccs_fit.py\n│\xa0\xa0         └── ccs_validate.py\n└── tests\n```\n\n* `ccs_fetch`           - Executable to construct the traning-set (structures.json) from a pre-existing ASE-database with DFT-data.\n* `ccs_fit`             - The primary executable file for the ccs package.\n* `main.py`             - A module to parse input files.\n* `objective.py`        - A module which contains the objective function and solver.\n* `spline_functions.py` - A module for spline construction/evaluation/output. \n\n<!---\n### Prerequisites\n\nYou need to install the following softwares\n\n```\npip install numpy\npip install scipy\npip install ase\npip install cvxopt\n```\n### Installing from source\n\n#### Git clone\n\n```\ngit clone git@github.com/Teoroo-CMC/CCS.git\ncd CCS\npython setup.py install\n```\n--->\n\n### (Recommended) installing from pip\npip install ccs_fit\n\n### Installing from source using poetry\n```\ngit clone https://github.com/Teoroo-CMC/CCS_fit.git ccs_fit\ncd ccs_fit\n\n# Install python package manager poetry (see https://python-poetry.org/docs/ for more explicit installation instructions)\ncurl -sSL https://install.python-poetry.org | python3 -\n# You might have to add poetry to your PATH\npoetry --version # to see if poetry installed correctly\npoetry install # to install ccs_fit\n```\n### Environment Variables\nSet the following environment variables:\n```\n$export PYTHONPATH=<path-to-CCS-package>:$PYTHONPATH\n$export PATH=<path-to-CCS-bin>:$PATH\n\nWithin a conda virtual environment, you can update the path by using:\nconda develop <path-to-CCS-package>\n```\n\n\n## Tutorials\n\nWe provide tutorials in the [examples](examples/) folder. To run the example, go to one of the folders. Each contain the neccesery input files required for the task at hand. A sample `CCS_input.json` for O2 is shown below:\n```\n{\n        "General": {\n                "interface": "CCS"\n        },\n        "Train-set": "structures.json",\n        "Twobody": {\n                "O-O": {\n                        "Rcut": 2.5,\n                        "Resolution": 0.02,\n                        "Swtype": "sw"\n                }\n        },\n        "Onebody": [\n                "O"\n        ]\n}\n\n```\nThe `CCS_input.json` file should provide at a minimum the block "General" specifying an interface. The default is to look for input structures in the file `structure.json` file. The format for `structure.json` is shown below :\n```\n{\n"energies":{\n        "S1": {\n                "Energy": -4.22425752,\n                "Atoms": {\n                        "O": 2\n                },\n                "O-O": [\n                        0.96\n                ]\n        },\n        "S2": {\n                "Energy": -5.29665634,\n                "Atoms": {\n                        "O": 2\n                },\n                "O-O": [\n                        0.98\n                ]\n        },\n        "S3": {\n                "Energy": -6.20910363,\n                "Atoms": {\n                        "O": 2\n                },\n                "O-O": [\n                        1.0\n                ]\n        },\n        "S4": {\n                "Energy": -6.98075271,\n                "Atoms": {\n                        "O": 2\n                },\n                "O-O": [\n                        1.02\n                ]\n        }\n}\n}\n```\nThe `structure.json` file contains different configurations labeled ("S1", "S2"...) and corresponding energy, pairwise distances (contained in an array labelled as "O-O" for oxygen). The stoichiometry of each configuration is given under the atoms label ("Atoms") as a key-value pair ("O" : 2 ). \n\n\nTo perform the fit : \n```\nccs_fit\n```\nThe following output files are obtained:\n```\nCCS_params.json CCS_error.out ccs.log \n```\n* CCS_params.json  - Contains the spline coefficients, and one-body terms for two body potentials.\n* error.out        - Contains target energies, predicted energies and absolute error for each configuration.\n* ccs.log          - Contains debug information\n## Authors\n\n* **Akshay Krishna AK** \n* **Jolla Kullgren** \n* **Eddie Wadbro** \n* **Peter Broqvist**\n* **Thijs Smolders**\n\n## Funding\nThis project has received funding from the European Union\'s Horizon 2020 research and innovation programme under grant agreement No 957189.\n\n## License\nThis project is licensed under the GPLv3 License - see the [LICENSE](LICENSE) file for details.\n\n## Acknowledgement\nWe thank all the members of  [TEOROO-group](http://www.teoroo.kemi.uu.se/) at Uppsala University, Sweden.\n',
    'author': 'Akshay Krishna AK',
    'author_email': 'None',
    'maintainer': 'Jolla Kullgren',
    'maintainer_email': 'jolla.kullgren@kemi.uu.se',
    'url': 'https://github.com/Teoroo-CMC/CCS',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9',
}


setup(**setup_kwargs)
